@@ -76,6 +76,20 @@ static struct {
   { 304, 190, 1400, 1050, "Hewlett-Packard", "HP Compaq nc6000 (PN887PA#ABG)", "F.0F" },
   { 332, 208, 1280,  800, "MICRO-STAR INT'L CO.,LTD.", "MS-1011", "0101" },
   { 304, 189, 1440,  900, "Dell Inc.", "Latitude D630C", NULL },
+  { 305, 229, 1024,  768, "IBM CORPORATION", "4840563", NULL },
+  { 337, 270, 1280, 1024, "IBM CORPORATION", "4840573", NULL },
+  { 305, 229, 1024,  768, "IBM CORPORATION", "4836135", NULL },
+  { 305, 229, 1024,  768, "IBM CORPORATION", "48361Z5", NULL },
+  { 305, 229, 1024,  768, "IBM CORPORATION", "48361z5", NULL },
+  { 305, 229, 1024,  768, "IBM CORPORATION", "48381Z5", NULL },
+  { 305, 229, 1024,  768, "IBM CORPORATION", "48381z5", NULL },
+  { 337, 270, 1280, 1024, "IBM CORPORATION", "4836137", NULL },
+  { 337, 270, 1280, 1024, "IBM CORPORATION", "48361Z7", NULL },
+  { 337, 270, 1280, 1024, "IBM CORPORATION", "48361z7", NULL },
+  { 337, 270, 1280, 1024, "IBM CORPORATION", "4838137", NULL },
+  { 337, 270, 1280, 1024, "IBM CORPORATION", "48381Z7", NULL },
+  { 337, 270, 1280, 1024, "IBM CORPORATION", "48381z7", NULL },
+  { 367, 230, 1440,  900, "Hewlett-Packard", "HP Pavilion dv7 Notebook PC", "F.12" },
 };
 
 #define BIOS_TEST
