@@ -49,7 +49,7 @@ void hd_scan_braille(hd_data_t *hd_data)
       hd->base_class.id == bc_comm &&
       hd->sub_class.id == sc_com_ser &&
       hd->unix_dev_name &&
-      !hd->tag.ser_skip &&
+      !hd->tag.skip_braille &&
       !has_something_attached(hd_data, hd)
     ) {
       cnt++;
