@@ -329,10 +329,11 @@ int match_modinfo(hd_data_t *hd_data, modinfo_t *db, modinfo_t *match)
         if(!strncmp(db->module, "pata_", sizeof "pata_" - 1)) {
           prio += hd_data->flags.pata ? 1 : -1;
         }
-        if(!strcmp(db->module, "piix")) {	/* ata_piix vs. piix */
+        if(!strcmp(db->module, "piix")) {		/* ata_piix vs. piix */
           prio += hd_data->flags.pata ? -1 : 1;
         }
         if(!strcmp(db->module, "generic")) prio -= 2;
+        if(!strcmp(db->module, "sk98lin")) prio -= 1;	/* deprecate sk98lin (#298724) */
       }
       break;
 
@@ -416,8 +417,17 @@ driver_info_t *hd_modinfo_db(hd_data_t *hd_data, modinfo_t *modinfo_db, hd_t *hd
         if(
           di2->any.type == di_module &&
           di2->any.hddb0 &&
-          di2->any.hddb0->str &&
-          !hd_mod_cmp(di2->any.hddb0->str, modinfo_db->module)
+          (
+            (
+              di2->any.hddb0->str &&
+              !hd_mod_cmp(di2->any.hddb0->str, modinfo_db->module)
+            ) ||
+            (
+              di2->any.hddb0->next &&
+              di2->any.hddb0->next->str &&
+              !hd_mod_cmp(di2->any.hddb0->next->str, modinfo_db->module)
+            )
+          )
         ) break;
       }
 
