@@ -67,8 +67,8 @@ void hd_scan_net(hd_data_t *hd_data)
 
   PROGRESS(1, 0, "get network data");
 
-  sf_class = reverse_str_list(read_dir("/sys/class/net", 'l'));
-  if(!sf_class) sf_class = reverse_str_list(read_dir("/sys/class/net", 'd'));
+  sf_class = read_dir("/sys/class/net", 'l');
+  if(!sf_class) sf_class = read_dir("/sys/class/net", 'd');
 
   if(!sf_class) {
     ADD2LOG("sysfs: no such class: net\n");
@@ -296,8 +296,11 @@ void hd_scan_net(hd_data_t *hd_data)
             add_res_entry(&hd_card->res, res);
           }
         }
-        /* add interface names */
-        add_if_name(hd_card, hd);
+        /*
+         * add interface names...
+         * but not wmasterX (bnc #441778)
+         */
+        if(if_type != 801) add_if_name(hd_card, hd);
       }
     }
 
@@ -354,12 +357,30 @@ void hd_scan_net(hd_data_t *hd_data)
   add_uml(hd_data);
   add_kma(hd_data);
 
-  /* add link status info */
+  /* add link status info & dump eeprom */
   for(hd = hd_data->hd ; hd; hd = hd->next) {
     if(
       hd->module == hd_data->module &&
       hd->base_class.id == bc_network_interface
     ) {
+      char *buf = NULL;
+      str_list_t *sl0, *sl;
+
+      if(hd_probe_feature(hd_data, pr_net_eeprom) && hd->unix_dev_name) {
+        PROGRESS(2, 0, "eeprom dump");
+
+        str_printf(&buf, 0, "|/usr/sbin/ethtool -e %s 2>/dev/null", hd->unix_dev_name);
+        if((sl0 = read_file(buf, 0, 0))) {
+          ADD2LOG("----- %s %s -----\n", hd->unix_dev_name, "EEPROM dump");
+          for(sl = sl0; sl; sl = sl->next) {
+            ADD2LOG("%s", sl->str);
+          }
+          ADD2LOG("----- %s end -----\n", "EEPROM dump");
+          free_str_list(sl0);
+        }
+        free(buf);
+      }
+
       for(res = hd->res; res; res = res->next) {
         if(res->any.type == res_link) break;
       }
