@@ -58,6 +58,7 @@ static void hd_read_of_platform(hd_data_t *hd_data);
 static void add_xen_network(hd_data_t *hd_data);
 static void add_xen_storage(hd_data_t *hd_data);
 static void hd_read_virtio(hd_data_t *hd_data);
+static void hd_read_ibmebus(hd_data_t *hd_data);
 
 void hd_scan_sysfs_pci(hd_data_t *hd_data)
 {
@@ -103,6 +104,9 @@ void hd_scan_sysfs_pci(hd_data_t *hd_data)
 
   PROGRESS(10, 0, "virtio");
   hd_read_virtio(hd_data);
+
+  PROGRESS(11, 0, "ibmebus");
+  hd_read_ibmebus(hd_data);
 }
 
 
@@ -124,7 +128,7 @@ void hd_pci_read_data(hd_data_t *hd_data)
   str_list_t *sf_bus, *sf_bus_e;
   char *sf_dev;
 
-  sf_bus = reverse_str_list(read_dir("/sys/bus/pci/devices", 'l'));
+  sf_bus = read_dir("/sys/bus/pci/devices", 'l');
 
   if(!sf_bus) {
     ADD2LOG("sysfs: no such bus: pci\n");
@@ -581,7 +585,7 @@ void hd_read_macio(hd_data_t *hd_data)
   str_list_t *sf_bus, *sf_bus_e;
   char *sf_dev;
 
-  sf_bus = reverse_str_list(read_dir("/sys/bus/macio/devices", 'l'));
+  sf_bus = read_dir("/sys/bus/macio/devices", 'l');
 
   if(!sf_bus) {
     ADD2LOG("sysfs: no such bus: macio\n");
@@ -685,7 +689,7 @@ void hd_read_vio(hd_data_t *hd_data)
   str_list_t *sf_bus, *sf_bus_e;
   char *sf_dev;
 
-  sf_bus = reverse_str_list(read_dir("/sys/bus/vio/devices", 'l'));
+  sf_bus = read_dir("/sys/bus/vio/devices", 'l');
 
   if(!sf_bus) {
     ADD2LOG("sysfs: no such bus: vio\n");
@@ -806,7 +810,7 @@ void hd_read_platform(hd_data_t *hd_data)
   char *sf_dev, *sf_eth_net;
   int mv643xx_eth_seen = 0;
 
-  sf_bus = reverse_str_list(read_dir("/sys/bus/platform/devices", 'l'));
+  sf_bus = read_dir("/sys/bus/platform/devices", 'l');
 
   if(!sf_bus) {
     ADD2LOG("sysfs: no such bus: platform\n");
@@ -858,7 +862,7 @@ void hd_read_of_platform(hd_data_t *hd_data)
   hd_t *hd;
   char *sf_dev;
 
-  sf_bus = reverse_str_list(read_dir("/sys/bus/of_platform/devices", 'l'));
+  sf_bus = read_dir("/sys/bus/of_platform/devices", 'l');
 
   if(!sf_bus) {
     ADD2LOG("sysfs: no such bus: of_platform\n");
@@ -958,7 +962,7 @@ void hd_read_ps3_system_bus(hd_data_t *hd_data)
   str_list_t *sf_bus, *sf_bus_e, *sf_eth_dev, *sf_eth_dev_e;
   char *sf_dev, *sf_eth_net, *sf_eth_wireless;
 
-  sf_bus = reverse_str_list(read_dir("/sys/bus/ps3_system_bus/devices", 'l'));
+  sf_bus = read_dir("/sys/bus/ps3_system_bus/devices", 'l');
 
   if(!sf_bus) {
     ADD2LOG("sysfs: no such bus: ps3_system_bus\n");
@@ -1085,6 +1089,92 @@ void hd_read_ps3_system_bus(hd_data_t *hd_data)
   free_str_list(sf_bus);
 }
 
+void hd_read_ibmebus(hd_data_t *hd_data)
+{
+  char *sf_dev, *s, *modalias;
+  str_list_t *sf_bus, *sf_bus_e;
+  hd_t *hd, *hd_ehea_base = NULL;
+  int ehea_active = 0;
+
+  sf_bus = read_dir("/sys/bus/ibmebus/devices", 'l');
+
+  if(!sf_bus) {
+    ADD2LOG("sysfs: no such bus: ibmebus\n");
+    return;
+  }
+
+  for(sf_bus_e = sf_bus; sf_bus_e; sf_bus_e = sf_bus_e->next) {
+    sf_dev = new_str(hd_read_sysfs_link("/sys/bus/ibmebus/devices", sf_bus_e->str));
+
+    ADD2LOG(
+      "  ibmebus device: name = %s\n    path = %s\n",
+      sf_bus_e->str,
+      hd_sysfs_id(sf_dev)
+    );
+
+    if((modalias = get_sysfs_attr_by_path(sf_dev, "modalias"))) {
+      modalias = canon_str(modalias, strlen(modalias));
+
+      ADD2LOG("    modalias = \"%s\"\n", modalias);
+
+      if(0);
+      else if(strstr(modalias, "Nlhea") && strstr(modalias, "CIBM,lhea")) {
+        /* ==> 23c00100.lhea/modalias <==
+	 * of:NlheaT<NULL>CIBM,lhea
+	 * ehea
+	 */
+        hd = hd_ehea_base = add_hd_entry(hd_data, __LINE__, 0);
+
+        hd->bus.id = bus_ibmebus;
+        hd->vendor.id = MAKE_ID(TAG_PCI, 0x1014); /* IBM */
+        hd->base_class.id = bc_network;
+        hd->sub_class.id = 0;	/* ethernet */
+        str_printf(&hd->device.name, 0, "IBM Host Ethernet Adapter");
+
+        hd->modalias = new_str(modalias);
+
+        hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+        hd->sysfs_bus_id = new_str(sf_bus_e->str);
+        s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
+        if(s) add_str_list(&hd->drivers, s);
+      }
+      else if(strstr(modalias, "Nethernet") && strstr(modalias, "CIBM,lhea-ethernet")) {
+        /* ==> port1/modalias <==
+	 * of:NethernetTnetworkCIBM,lhea-ethernet
+	 * eth1
+	 */
+        hd = add_hd_entry(hd_data, __LINE__, 0);
+        ehea_active = 1;
+        hd->bus.id = bus_ibmebus;
+        hd->modalias = new_str(modalias);
+        hd->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+        hd->sysfs_bus_id = new_str(sf_bus_e->str);
+        hd->vendor.id = MAKE_ID(TAG_PCI, 0x1014); /* IBM */
+        hd->base_class.id = bc_network;
+        hd->sub_class.id = 0;	/* ethernet */
+        // to get proper module info
+        hd->compat_vendor.id = MAKE_ID(TAG_SPECIAL, 0x0403);
+        hd->compat_device.id = MAKE_ID(TAG_SPECIAL, 0x0001);
+	s = strpbrk(hd->sysfs_bus_id, "0123456789");
+	if(s) {
+          hd->slot = strtol(s, NULL, 10);
+          str_printf(&hd->device.name, 0, "IBM Host Ethernet Adapter Port %d", hd->slot);
+	}
+
+        s = hd_sysfs_find_driver(hd_data, hd->sysfs_id, 1);
+        add_str_list(&hd->drivers, s ?: "ehea");
+      }
+
+      modalias = free_mem(modalias);
+    }
+    free_mem(sf_dev);
+  }
+
+  if(hd_ehea_base && ehea_active) {
+    // remove it if we have the real devices
+    hd_ehea_base->base_class.id = bc_none;
+  }
+}
 
 /*
  * Get xen (network & storage) data from sysfs.
@@ -1098,7 +1188,7 @@ void hd_read_xen(hd_data_t *hd_data)
   char *sf_dev, *drv, *module;
   unsigned u;
 
-  sf_bus = reverse_str_list(read_dir("/sys/bus/xen/devices", 'l'));
+  sf_bus = read_dir("/sys/bus/xen/devices", 'l');
 
   if(!sf_bus) {
     ADD2LOG("sysfs: no such bus: xen\n");
@@ -1240,7 +1330,7 @@ void hd_read_vm(hd_data_t *hd_data)
   str_list_t *sf_bus, *sf_bus_e;
   char *sf_dev, *drv, *drv_name;
 
-  sf_bus = reverse_str_list(read_dir("/sys/bus/vmbus/devices", 'l'));
+  sf_bus = read_dir("/sys/bus/vmbus/devices", 'l');
 
   if(!sf_bus) {
     ADD2LOG("sysfs: no such bus: vm\n");
@@ -1305,7 +1395,7 @@ void hd_read_virtio(hd_data_t *hd_data)
   str_list_t *sf_bus, *sf_bus_e;
   char *sf_dev, *drv, *drv_name, *modalias;
 
-  sf_bus = reverse_str_list(read_dir("/sys/bus/virtio/devices", 'l'));
+  sf_bus = read_dir("/sys/bus/virtio/devices", 'l');
 
   if(!sf_bus) {
     ADD2LOG("sysfs: no such bus: virtio\n");
