@@ -29,9 +29,6 @@ static void read_str(char *path, char *name, char **str);
 static void read_mem(char *path, char *name, unsigned char **mem, unsigned len);
 static void read_int(char *path, char *name, int *val);
 static void read_devtree(hd_data_t *hd_data);
-static void add_legacy_prom_devices(hd_data_t *hd_data, devtree_t *dt);
-static void add_prom_ehea(hd_data_t *hd_data, devtree_t *dt);
-static void add_devices(hd_data_t *hd_data);
 static void dump_devtree_data(hd_data_t *hd_data);
 
 static unsigned veth_cnt, vscsi_cnt;
@@ -373,7 +370,6 @@ void hd_scan_prom(hd_data_t *hd_data)
     fclose(f);
   }
   if(hd_data->debug) dump_devtree_data(hd_data);
-  add_devices(hd_data);
 
   PROGRESS(2, 0, "color");
 
@@ -542,80 +538,6 @@ void read_devtree(hd_data_t *hd_data)
 
 }
 
-void add_legacy_prom_devices(hd_data_t *hd_data, devtree_t *dt)
-{
-  if(dt->pci) return;
-
-  add_prom_ehea(hd_data, dt);
-}
-
-void add_prom_ehea(hd_data_t *hd_data, devtree_t *dt)
-{
-  hd_t *hd;
-  hd_res_t *res;
-  char *path = NULL;
-  unsigned char *hw_addr_bin = NULL;
-  char *hw_addr = NULL;
-  int slot;
-
-  if(
-    dt->device_type &&
-    dt->compatible &&
-    !strcmp(dt->device_type, "network") &&
-    !strcmp(dt->compatible, "IBM,lhea-ethernet")
-  ) {
-    str_printf(&path, 0, PROC_PROM "/%s", dt->path);
-
-    ADD2LOG("  ehea: %s\n", path);
-
-    hd = add_hd_entry(hd_data, __LINE__, 0);
-    hd->bus.id = bus_none;
-    hd->base_class.id = bc_network;
-    hd->sub_class.id = 0;	/* ethernet */
-
-    hd->vendor.id = MAKE_ID(TAG_SPECIAL, 0x6001);
-    hd->device.id = MAKE_ID(TAG_SPECIAL, 0x1003);
-    hd->rom_id = new_str(dt->path);
-
-    read_int(path, "ibm,hea-port-no", &slot);
-    hd->slot = slot;
-
-    read_str(path, "ibm,fw-adapter-name", &hd->device.name);
-    if(!hd->device.name) {
-      hd->device.name = new_str("IBM Host Ethernet Adapter");
-    }
-
-    // "mac-address" or "local-mac-address" ?
-    read_mem(path, "local-mac-address", &hw_addr_bin, 6);
-
-    if(hw_addr_bin) {
-      str_printf(
-        &hw_addr, 0, "%02x:%02x:%02x:%02x:%02x:%02x",
-        hw_addr_bin[0], hw_addr_bin[1],
-        hw_addr_bin[2], hw_addr_bin[3],
-        hw_addr_bin[4], hw_addr_bin[5]
-      );
-      res = new_mem(sizeof *res);
-      res->hwaddr.type = res_hwaddr;
-      res->hwaddr.addr = new_str(hw_addr);
-      add_res_entry(&hd->res, res);
-    }
-  }
-
-  free_mem(hw_addr_bin);
-  free_mem(hw_addr);
-  free_mem(path);
-}
-
-void add_devices(hd_data_t *hd_data)
-{
-  devtree_t *dt;
-
-  for(dt = hd_data->devtree; dt; dt = dt->next) {
-    if(!dt->pci) add_legacy_prom_devices(hd_data, dt);
-  }
-}
-
 void dump_devtree_data(hd_data_t *hd_data)
 {
   unsigned u;
