@@ -91,7 +91,7 @@ void get_usb_devs(hd_data_t *hd_data)
   str_list_t *sf_bus, *sf_bus_e;
   char *sf_dev, *sf_dev_2;
 
-  sf_bus = reverse_str_list(read_dir("/sys/bus/usb/devices", 'l'));
+  sf_bus = read_dir("/sys/bus/usb/devices", 'l');
 
   if(!sf_bus) {
     ADD2LOG("sysfs: no such bus: usb\n");
@@ -519,6 +519,10 @@ void add_input_dev(hd_data_t *hd_data, char *name)
     }
     ADD2LOG("    dev = %u:%u\n", u1, u2);
   }
+  else {
+    ADD2LOG("    no dev - ignored\n");
+    return;
+  }
 
   sf_dev = new_str(hd_read_sysfs_link(name, "device"));
 
@@ -541,7 +545,7 @@ void add_input_dev(hd_data_t *hd_data, char *name)
     }
 
     bus_name = NULL;
-    if((s = hd_read_sysfs_link(sf_dev, "bus"))) {
+    if((s = hd_read_sysfs_link(sf_dev, "subsystem"))) {
       bus_name = strrchr(s, '/');
       if(bus_name) bus_name++;
       bus_name = new_str(bus_name);
@@ -614,9 +618,9 @@ void get_input_devs(hd_data_t *hd_data)
    * A bit tricky: if there are links, assume newer sysfs layout with compat
    * symlinks; if not, assume old layout with directories.
    */
-  sf_dir = reverse_str_list(read_dir("/sys/class/input", 'l'));
+  sf_dir = read_dir("/sys/class/input", 'l');
   if(!sf_dir) {
-    sf_dir = reverse_str_list(read_dir("/sys/class/input", 'd'));
+    sf_dir = read_dir("/sys/class/input", 'd');
     is_dir = 1;
   }
   
@@ -652,8 +656,8 @@ void get_printer_devs(hd_data_t *hd_data)
   char *sf_cdev = NULL, *sf_dev;
   char *sf_drv_name, *sf_drv, *bus_id, *bus_name;
 
-  sf_class = reverse_str_list(read_dir("/sys/class/usb", 'D'));
-  if(!sf_class) sf_class = reverse_str_list(read_dir("/sys/class/usb_endpoint", 'l'));
+  sf_class = read_dir("/sys/class/usb", 'D');
+  if(!sf_class) sf_class = read_dir("/sys/class/usb_endpoint", 'l');
 
   if(!sf_class) {
     ADD2LOG("sysfs: no such class: usb\n");
@@ -821,7 +825,7 @@ void get_serial_devs(hd_data_t *hd_data)
   char *sf_cdev = NULL, *sf_dev;
   char *sf_drv_name, *sf_drv, *bus_id, *bus_name;
 
-  sf_class = reverse_str_list(read_dir("/sys/class/tty", 'D'));
+  sf_class = read_dir("/sys/class/tty", 'D');
 
   if(!sf_class) {
     ADD2LOG("sysfs: no such class: tty\n");
@@ -903,7 +907,7 @@ void get_serial_devs(hd_data_t *hd_data)
             hd->vendor.id == MAKE_ID(TAG_USB, 0x0403) &&
             hd->device.id == MAKE_ID(TAG_USB, 0x6001)
           ) {
-            hd->tag.ser_skip = 1;
+            hd->tag.skip_mouse = hd->tag.skip_modem = 1;
           }
         }
       }
